@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2017 - 2020 Avram Lubkin, All Rights Reserved
+# Copyright 2017 - 2021 Avram Lubkin, All Rights Reserved
 
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -18,6 +18,8 @@ import re
 import sys
 import time
 
+from prefixed import Float
+
 from enlighten._basecounter import BaseCounter, PrintableCounter
 from enlighten._util import (EnlightenWarning, FORMAT_MAP_SUPPORT, format_time,
                              raise_from_none, warn_best_level)
@@ -45,9 +47,11 @@ except (AttributeError, TypeError):  # pragma: no cover(Non-standard Terminal)
     pass
 
 # Reserved fields
-COUNTER_FIELDS = {'count', 'desc', 'desc_pad', 'elapsed', 'interval', 'rate', 'unit', 'unit_pad',
-                  'bar', 'eta', 'len_total', 'percentage', 'total', 'fill'}
-RE_SUBCOUNTER_FIELDS = re.compile(r'(?:count|percentage|eta|interval|rate)_\d+')
+BAR_SPECIFIC_FIELDS = {'bar', 'eta', 'len_total', 'percentage'}
+COUNTER_SPECIFIC_FIELDS = {'fill'}
+RESERVED_FIELDS = {'count', 'desc', 'desc_pad', 'elapsed', 'interval', 'rate', 'unit', 'unit_pad',
+                   'total'} | BAR_SPECIFIC_FIELDS | COUNTER_SPECIFIC_FIELDS
+RE_SUBCOUNTER_FIELDS = re.compile(r'(count|percentage|eta|interval|rate)_(\d+)')
 
 
 class SubCounter(BaseCounter):
@@ -149,7 +153,15 @@ class Counter(PrintableCounter):
         seagreen
         peru
 
+    .. _Prefixed documentation: https://prefixed.readthedocs.io/en/stable/index.html
+    .. _format specifiers: https://prefixed.readthedocs.io/en/stable/format_spec.html
+    .. _SI (metric): https://en.wikipedia.org/wiki/Metric_prefix
+    .. _IEC (binary): https://en.wikipedia.org/wiki/Binary_prefix
+
+
     Args:
+        all_fields(bool): Populate ``rate``, ``interval``, and ``eta``
+            formatting fields in subcounters
         bar_format(str): Progress bar format, see :ref:`Format <counter_format>` below
         count(int): Initial count (Default: 0)
         counter_format(str): Counter format, see :ref:`Format <counter_format>` below
@@ -288,15 +300,16 @@ class Counter(PrintableCounter):
             # Example output
             'Processing    22%|█████▊                   |  23/101 [00:27<01:32, 0.84 Files/s]'
 
-
         Available fields:
 
         - count(:py:class:`int`) - Current value of ``count``
         - desc(:py:class:`str`) - Value of ``desc``
         - desc_pad(:py:class:`str`) - A single space if ``desc`` is set, otherwise empty
         - elapsed(:py:class:`str`) - Time elapsed since instance was created
-        - interval(:py:class:`float`) - Average seconds per iteration (inverse of rate)
-        - rate(:py:class:`float`) - Average iterations per second since instance was created
+        - interval(:py:class:`prefixed.Float`) - Average seconds per iteration (inverse of rate)
+        - rate(:py:class:`prefixed.Float`) - Average iterations per second
+          since instance was created
+        - total(:py:class:`int`) - Value of ``total``
         - unit(:py:class:`str`) - Value of ``unit``
         - unit_pad(:py:class:`str`) - A single space if ``unit`` is set, otherwise empty
 
@@ -306,7 +319,6 @@ class Counter(PrintableCounter):
         - eta(:py:class:`str`) - Estimated time to completion
         - len_total(:py:class:`int`) - Length of ``total`` when converted to a string
         - percentage(:py:class:`float`) - Percentage complete
-        - total(:py:class:`int`) - Value of ``total``
 
         Additional fields for ``counter_format`` only:
 
@@ -315,13 +327,21 @@ class Counter(PrintableCounter):
 
         Additional fields when subcounters are used:
 
-        - count_n (:py:class:`int`) - Current value of ``count``
+        - count_n(:py:class:`int`) - Current value of ``count``
         - count_0(:py:class:`int`) - Remaining count after deducting counts for all subcounters
-        - count_00 (:py:class:`int`) - Sum of counts from all subcounters
-        - percentage_n (:py:class:`float`) - Percentage complete (``bar_format`` only)
+        - count_00(:py:class:`int`) - Sum of counts from all subcounters
+        - interval_0(:py:class:`prefixed.Float`) - Average seconds per non-subcounter iteration
+          (inverse of rate_0)
+        - interval_00(:py:class:`prefixed.Float`) - Average seconds per iteration for all
+          subcounters (inverse of rate_00)
+        - percentage_n(:py:class:`float`) - Percentage complete (``bar_format`` only)
         - percentage_0(:py:class:`float`) - Remaining percentage after deducting percentages
           for all subcounters (``bar_format`` only)
-        - percentage_00 (:py:class:`float`) - Total of percentages from all subcounters
+        - percentage_00(:py:class:`float`) - Total of percentages from all subcounters
+        - rate_0(:py:class:`prefixed.Float`) - Average iterations per second excluding subcounters
+          since instance was created
+        - rate_00(:py:class:`prefixed.Float`) - Average iterations per second of all subcounters
+          since instance was created
 
         .. note::
 
@@ -333,8 +353,22 @@ class Counter(PrintableCounter):
         ``all_fields`` set to :py:data:`True`:
 
         - eta_n (:py:class:`str`) - Estimated time to completion (``bar_format`` only)
-        - interval_n(:py:class:`float`) - Average seconds per iteration (inverse of rate)
-        - rate_n (:py:class:`float`) - Average iterations per second since parent was created
+        - interval_n(:py:class:`prefixed.Float`) - Average seconds per iteration (inverse of rate)
+        - rate_n (:py:class:`prefixed.Float`) - Average iterations per second
+          since parent was created
+
+        .. note::
+
+            ``count`` and ``total`` fields, including ``count_0``, ``count_00``, and ``count_n``,
+            default to :py:class:`int`. If :py:attr:`~Counter.total` or or :py:attr:`~Counter.count`
+            are set to a :py:class:`float`, or a :py:class:`float` is provided to
+            :py:meth:`~Counter.update`, these fields will be :py:class:`prefixed.Float` instead.
+
+            This allows additional `format specifiers`_ using `SI (metric)`_ and `IEC (binary)`_
+            prefixes. See the `Prefixed documentation`_ for more details.
+
+            This will also require a custom format and affect the accuracy of the ``len_total``
+            field.
 
         User-defined fields:
 
@@ -399,7 +433,7 @@ class Counter(PrintableCounter):
     """
     # pylint: disable=too-many-instance-attributes
 
-    __slots__ = ('bar_format', 'counter_format', 'desc', 'fields', 'manager',
+    __slots__ = ('all_fields', 'bar_format', 'counter_format', 'desc', 'fields', 'manager',
                  'offset', 'series', 'total', 'unit', '_fields', '_subcounters')
     _repr_attrs = ('desc', 'total', 'count', 'unit', 'color')
 
@@ -410,6 +444,8 @@ class Counter(PrintableCounter):
 
         # Accept additional_fields for backwards compatibility
         self.fields = kwargs.pop('fields', kwargs.pop('additional_fields', {}))
+
+        self.all_fields = kwargs.pop('all_fields', False)
         self.bar_format = kwargs.pop('bar_format', BAR_FMT)
         self.counter_format = kwargs.pop('counter_format', COUNTER_FMT)
         self.desc = kwargs.pop('desc', None)
@@ -440,7 +476,8 @@ class Counter(PrintableCounter):
 
         return sum(subcounter.count for subcounter in self._subcounters)
 
-    def _get_subcounters(self, elapsed, bar_fields=True):
+    # pylint: disable=too-many-locals
+    def _get_subcounters(self, elapsed, fields, bar_fields=True, force_float=False):
         """
         Args:
             elapsed(float): Time since started.
@@ -456,15 +493,23 @@ class Counter(PrintableCounter):
         percentage will be set to 0.0
         """
 
-        fields = {}
         subcounters = []
+        count_00 = 0
+        start_count_00 = 0
+
+        if not self._subcounters:
+            return subcounters
 
         for num, subcounter in enumerate(self._subcounters, 1):
 
-            fields['count_%d' % num] = subcounter.count
+            count = subcounter.count
+            count_00 += count
+            start_count_00 += subcounter.start_count
+
+            fields['count_%d' % num] = Float(count) if force_float else count
 
             if self.total and bar_fields:
-                subPercentage = subcounter.count / float(self.total)
+                subPercentage = count / float(self.total)
             else:
                 subPercentage = 0.0
 
@@ -474,29 +519,44 @@ class Counter(PrintableCounter):
             # Save in tuple: count, percentage
             subcounters.append((subcounter, subPercentage))
 
-            if subcounter.all_fields:
+            if not subcounter.all_fields:
+                continue
 
-                interations = float(abs(subcounter.count - subcounter.start_count))
+            # Explicit conversion to float required for Python 2
+            interations = float(abs(count - subcounter.start_count))
+            rate = Float(interations / elapsed) if elapsed else Float(0.0)
+            fields['rate_%d' % num] = rate
+            fields['interval_%d' % num] = rate ** -1 if rate else rate
 
-                if elapsed:
-                    # Use float to force to float in Python 2
-                    rate = fields['rate_%d' % num] = interations / elapsed
-                else:
-                    rate = fields['rate_%d' % num] = 0.0
+            if not bar_fields:
+                continue
 
-                fields['interval_%d' % num] = rate ** -1 if rate else 0.0
+            if self.total == 0:
+                fields['eta_%d' % num] = u'00:00'
+            elif rate:
+                fields['eta_%d' % num] = format_time((self.total - interations) / rate)
+            else:
+                fields['eta_%d' % num] = u'?'
 
-                if not bar_fields:
-                    continue
+        # Percentage_0 and percentage_00, bar_format only
+        if bar_fields:
+            fields['percentage_00'] = percentage_00 = sum(sub[1] for sub in subcounters) * 100
+            fields['percentage_0'] = fields['percentage'] - percentage_00
 
-                if self.total == 0:
-                    fields['eta_%d' % num] = u'00:00'
-                elif rate:
-                    fields['eta_%d' % num] = format_time((self.total - interations) / rate)
-                else:
-                    fields['eta_%d' % num] = u'?'
+        # count_00 fields (Sum of subcounters)
+        fields['count_00'] = Float(count_00) if force_float else count_00
+        rate = Float(float(abs(count_00 - start_count_00)) / elapsed) if elapsed else Float(0.0)
+        fields['rate_00'] = rate
+        fields['interval_00'] = rate ** -1 if rate else rate
 
-        return subcounters, fields
+        # count_0 fields (Excluding subcounters)
+        count_0 = fields['count_0'] = fields['count'] - count_00
+        start_count_0 = self.start_count - start_count_00
+        rate = Float(float(abs(count_0 - start_count_0)) / elapsed) if elapsed else Float(0.0)
+        fields['rate_0'] = rate
+        fields['interval_0'] = rate ** -1 if rate else rate
+
+        return subcounters
 
     # pylint: disable=too-many-locals,too-many-branches,too-many-statements
     def format(self, width=None, elapsed=None):
@@ -512,6 +572,7 @@ class Counter(PrintableCounter):
         """
 
         width = width or self.manager.width
+        total = self.total
 
         iterations = float(abs(self.count - self.start_count))
 
@@ -519,21 +580,24 @@ class Counter(PrintableCounter):
         fields.update(self._fields)
 
         # Warn on reserved fields
-        reserved_fields = (set(fields) & COUNTER_FIELDS) | set(
-            match.group() for match in (RE_SUBCOUNTER_FIELDS.match(key) for key in fields) if match
-        )
+        reserved_fields = set(fields) & RESERVED_FIELDS | {
+            match.group()
+            for match in (RE_SUBCOUNTER_FIELDS.match(key) for key in fields)
+            if match
+        }
+
         if reserved_fields:
             warn_best_level('Ignoring reserved fields specified as user-defined fields: %s' %
                             ', '.join(reserved_fields),
                             EnlightenWarning)
 
-        fields.update({'bar': u'{0}',
-                       'count': self.count,
-                       'desc': self.desc or u'',
-                       'total': self.total,
-                       'unit': self.unit or u'',
-                       'desc_pad': u' ' if self.desc else u'',
-                       'unit_pad': u' ' if self.unit else u''})
+        force_float = isinstance(self.count, float) or isinstance(total, float)
+        fields['count'] = Float(self.count) if force_float else self.count
+        fields['desc'] = self.desc or u''
+        fields['total'] = Float(total) if force_float and total is not None else total
+        fields['unit'] = self.unit or u''
+        fields['desc_pad'] = u' ' if self.desc else u''
+        fields['unit_pad'] = u' ' if self.unit else u''
 
         # Get elapsed time
         if elapsed is None:
@@ -542,22 +606,70 @@ class Counter(PrintableCounter):
         fields['elapsed'] = format_time(elapsed)
 
         # Get rate. Elapsed could be 0 if counter was not updated and has a zero total.
-        if elapsed:
-            # Use iterations so a counter running backwards is accurate
-            rate = fields['rate'] = iterations / elapsed
-        else:
-            rate = fields['rate'] = 0.0
-
-        fields['interval'] = rate ** -1 if rate else 0.0
+        rate = Float(iterations / elapsed) if elapsed else Float(0.0)
+        fields['rate'] = rate
+        fields['interval'] = rate ** -1 if rate else rate
 
         # Only process bar if total was given and n doesn't exceed total
-        if self.total is not None and self.count <= self.total:
-            return self._format_bar(fields, iterations, width, elapsed)
+        if total is not None and self.count <= total:
+            return self._format_bar(fields, iterations, width, elapsed, force_float)
 
         # Otherwise return a counter
-        return self._format_counter(fields, width, elapsed)
+        return self._format_counter(fields, width, elapsed, force_float)
 
-    def _format_bar(self, fields, iterations, width, elapsed):
+    def _get_format_error(self, field, bar_fields=True):
+        """
+        Args:
+            field (str): Field unavailable for formatting
+
+        Returns:
+            :py:class:`str`: Formatted error message
+
+        Creates an appropriate error message for a missing field in order to best inform user
+        """
+
+        match = RE_SUBCOUNTER_FIELDS.match(field)
+        subcounter = 0
+
+        reserve_msg = "Reserve field '%s' specified in format, but " % field
+
+        # Check subcounter fields
+        if match:
+
+            # Get subcounter number
+            subcounter = int(match.group(2))
+
+            # Base counter fields
+            if subcounter == 0:
+                msg = reserve_msg + 'no subcounters are configured'
+
+            # Subcounter not defined
+            elif subcounter > len(self._subcounters):
+                msg = reserve_msg + 'subcounter %d is not defined' % subcounter
+
+            # Bar-specific subcounter fields
+            elif match.group(1) in BAR_SPECIFIC_FIELDS and not bar_fields:
+                msg = reserve_msg + 'unavailable for counter_format'
+
+            # all_fields required for field
+            else:
+                msg = reserve_msg + "'all_fields' not specified for subcounter"
+
+        # Bar field provided to counter_format
+        elif field in BAR_SPECIFIC_FIELDS and not bar_fields:
+            msg = reserve_msg + 'unavailable for counter_format'
+
+        # Counter field provided to bar_format
+        elif field in COUNTER_SPECIFIC_FIELDS and bar_fields:
+            msg = reserve_msg + 'unavailable for bar_format'
+
+        # Not a reserved field
+        else:
+            msg = "'%s' specified in format, but not provided" % field
+
+        return msg % {'field': field, 'subcounter': subcounter}
+
+    def _format_bar(self, fields, iterations, width, elapsed, force_float):
         """
         Args:
             fields (dict): Initial set of formatting fields
@@ -571,6 +683,7 @@ class Counter(PrintableCounter):
         Format progress bar
         """
 
+        fields['bar'] = u'{0}'
         fields['len_total'] = len(str(self.total))
 
         # Get percentage
@@ -593,19 +706,7 @@ class Counter(PrintableCounter):
         fields['percentage'] = percentage * 100
 
         # Have to go through subcounters here so the fields are available
-        subcounters, subFields = self._get_subcounters(elapsed)
-
-        # Calculate count and percentage for remainder
-        count_0 = self.count
-        percentage_0 = percentage
-        if subcounters:
-            fields.update(subFields)
-            subcount = fields['count_00'] = sum(sub[0].count for sub in subcounters)
-            count_0 = fields['count_0'] = count_0 - subcount
-            subpercentage = sum(sub[1] for sub in subcounters)
-            fields['percentage_00'] = subpercentage * 100
-            percentage_0 = percentage - subpercentage
-            fields['percentage_0'] = percentage_0 * 100
+        subcounters = self._get_subcounters(elapsed, fields, force_float=force_float)
 
         # Partially format
         try:
@@ -614,7 +715,7 @@ class Counter(PrintableCounter):
             else:  # pragma: no cover
                 rtn = self.bar_format.format(**fields)
         except KeyError as e:
-            raise_from_none(ValueError('%r specified in format, but not provided' % e.args[0]))
+            raise_from_none(ValueError(self._get_format_error(e.args[0])))
 
         # Determine bar width
         if self.offset is None:
@@ -628,8 +729,8 @@ class Counter(PrintableCounter):
         barText = u''
 
         if subcounters:
-            block_count = [int(barWidth * percentage_0)]
-            partial_len = (barLen - 1) if count_0 else barLen
+            block_count = [int(barWidth * fields['percentage_0'] / 100)]
+            partial_len = (barLen - 1) if fields['count_0'] else barLen
             remaining = []
 
             # Get full blocks for subcounters and preserve remainders
@@ -662,14 +763,14 @@ class Counter(PrintableCounter):
 
         # If bar isn't complete, add partial block and fill
         if barLen < barWidth:
-            if count_0:
+            if fields.get('count_0', self.count):
                 barText += self.series[int(round((complete - barLen) * (len(self.series) - 1)))]
                 partial_len += 1
             barText += self.series[0] * (barWidth - partial_len)
 
         return rtn.format(self._colorize(barText))
 
-    def _format_counter(self, fields, width, elapsed):
+    def _format_counter(self, fields, width, elapsed, force_float):
         """
         Args:
             fields (dict): Initial set of formatting fields
@@ -682,13 +783,10 @@ class Counter(PrintableCounter):
         Format counter
         """
 
-        # Update fields from subcounters
         fields['fill'] = u'{0}'
-        subcounters, subFields = self._get_subcounters(elapsed, bar_fields=False)
-        if subcounters:
-            fields.update(subFields)
-            subcount = fields['count_00'] = sum(sub[0].count for sub in subcounters)
-            fields['count_0'] = self.count - subcount
+
+        # Update fields from subcounters
+        self._get_subcounters(elapsed, fields, bar_fields=False, force_float=force_float)
 
         try:
             if FORMAT_MAP_SUPPORT:
@@ -696,7 +794,7 @@ class Counter(PrintableCounter):
             else:  # pragma: no cover
                 rtn = self.counter_format.format(**fields)
         except KeyError as e:
-            raise_from_none(ValueError('%r specified in format, but not provided' % e.args[0]))
+            raise_from_none(ValueError(self._get_format_error(e.args[0], bar_fields=False)))
 
         return self._fill_text(rtn, width, offset=self.offset)
 
@@ -721,13 +819,13 @@ class Counter(PrintableCounter):
                     currentTime - self.last_update >= self.min_delta:
                 self.refresh(elapsed=currentTime - self.start)
 
-    def add_subcounter(self, color, count=0, all_fields=False):
+    def add_subcounter(self, color, count=0, all_fields=None):
         """
     Args:
         color(str): Series color as a string or RGB tuple see :ref:`Series Color <series_color>`
         count(int): Initial count (Default: 0)
         all_fields(bool): Populate ``rate``, ``interval``, and ``eta``
-            formatting fields (Default: False)
+            formatting fields (Default: False unless specified in parent)
 
     Returns:
         :py:class:`SubCounter`: Subcounter instance
@@ -735,6 +833,9 @@ class Counter(PrintableCounter):
     Add a subcounter for multicolored progress bars
         """
 
+        if all_fields is None:
+            all_fields = self.all_fields
+
         subcounter = SubCounter(self, color=color, count=count, all_fields=all_fields)
         self._subcounters.append(subcounter)
         return subcounter
