@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2017 - 2020 Avram Lubkin, All Rights Reserved
+# Copyright 2017 - 2021 Avram Lubkin, All Rights Reserved
 
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -271,26 +271,41 @@ class TestCounter(TestCase):
         subcounter2.count = 4
         subcounter3 = self.ctr.add_subcounter('white', count=1, all_fields=True)
 
-        subcounters, fields = self.ctr._get_subcounters(8)
-
+        fields = {'count': self.ctr.count, 'percentage': 60.0}
+        subcounters = self.ctr._get_subcounters(8, fields)
         self.assertEqual(subcounters, [(subcounter1, 0.0), (subcounter2, 0.4), (subcounter3, 0.1)])
-        self.assertEqual(fields, {'percentage_1': 0.0, 'percentage_2': 40.0, 'percentage_3': 10.0,
+        self.assertEqual(fields, {'count': 6, 'count_0': 1, 'count_00': 5,
+                                  'percentage': 60.0, 'percentage_0': 10.0, 'percentage_00': 50.0,
+                                  'percentage_1': 0.0, 'percentage_2': 40.0, 'percentage_3': 10.0,
                                   'count_1': 0, 'count_2': 4, 'count_3': 1,
+                                  'interval_0': 4.0, 'interval_00': 2.0,
                                   'interval_2': 2.0, 'interval_3': 0.0,
+                                  'rate_0': 0.25, 'rate_00': 0.5,
                                   'rate_2': 0.5, 'eta_2': '00:12', 'rate_3': 0.0, 'eta_3': '?'})
 
-        subcounters, fields = self.ctr._get_subcounters(0)
+        fields = {'count': self.ctr.count, 'percentage': 60.0}
+        subcounters = self.ctr._get_subcounters(0, fields)
         self.assertEqual(subcounters, [(subcounter1, 0.0), (subcounter2, 0.4), (subcounter3, 0.1)])
-        self.assertEqual(fields, {'percentage_1': 0.0, 'percentage_2': 40.0, 'percentage_3': 10.0,
+        self.assertEqual(fields, {'count': 6, 'count_0': 1, 'count_00': 5,
+                                  'percentage': 60.0, 'percentage_0': 10.0, 'percentage_00': 50.0,
+                                  'percentage_1': 0.0, 'percentage_2': 40.0, 'percentage_3': 10.0,
                                   'count_1': 0, 'count_2': 4, 'count_3': 1,
+                                  'interval_0': 0.0, 'interval_00': 0.0,
                                   'interval_2': 0.0, 'interval_3': 0.0,
+                                  'rate_0': 0.0, 'rate_00': 0.0,
                                   'rate_2': 0.0, 'eta_2': '?', 'rate_3': 0.0, 'eta_3': '?'})
 
         self.ctr = Counter(total=0, desc='Test', unit='ticks', manager=self.manager)
         subcounter1 = self.ctr.add_subcounter('red', all_fields=True)
-        subcounters, fields = self.ctr._get_subcounters(8)
+
+        fields = {'count': self.ctr.count, 'percentage': 0.0}
+        subcounters = self.ctr._get_subcounters(8, fields)
         self.assertEqual(subcounters, [(subcounter1, 0.0)])
-        self.assertEqual(fields, {'percentage_1': 0.0, 'count_1': 0,
+        self.assertEqual(fields, {'count': 0, 'count_0': 0, 'count_00': 0,
+                                  'percentage': 0.0, 'percentage_0': 0.0, 'percentage_00': 0.0,
+                                  'percentage_1': 0.0, 'count_1': 0,
+                                  'interval_0': 0.0, 'interval_00': 0.0,
+                                  'rate_0': 0.0, 'rate_00': 0.0,
                                   'interval_1': 0.0, 'rate_1': 0.0, 'eta_1': '00:00'})
 
     def test_get_subcounter_counter_format(self):
@@ -300,10 +315,14 @@ class TestCounter(TestCase):
         subcounter2.count = 6
         subcounter3 = self.ctr.add_subcounter('white', count=1, all_fields=True)
 
-        subcounters, fields = self.ctr._get_subcounters(8, bar_fields=False)
+        fields = {'count': self.ctr.count}
+        subcounters = self.ctr._get_subcounters(8, fields, bar_fields=False)
         self.assertEqual(subcounters, [(subcounter1, 0.0), (subcounter2, 0.0), (subcounter3, 0.0)])
-        self.assertEqual(fields, {'count_1': 0, 'count_2': 6, 'count_3': 1,
+        self.assertEqual(fields, {'count': 12, 'count_0': 5, 'count_00': 7,
+                                  'count_1': 0, 'count_2': 6, 'count_3': 1,
+                                  'interval_0': 0.75 ** -1, 'interval_00': 0.75 ** -1,
                                   'interval_2': 0.75 ** -1, 'interval_3': 0.0,
+                                  'rate_0': 0.75, 'rate_00': 0.75,
                                   'rate_2': 0.75, 'rate_3': 0.0})
 
     def test_remove(self):
@@ -539,6 +558,27 @@ class TestCounter(TestCase):
         self.assertRegex(formatted, r'Test  50%\|' + u'█+' +
                          r'[ ]+\| 50.1/100.2 \[00:5\d<00:5\d, \d.\d\d ticks/s\]')
 
+    def test_floats_prefixed(self):
+        """
+        Floats should support prefixed formatting
+        """
+        bar_format = u'{count:!.2j}B / {total:!.2j}B | {rate:!.2j}B/s | {interval:!.2j} s/B'
+
+        ctr = Counter(stream=self.tty.stdout, total=3.2 * 2 ** 20, bar_format=bar_format)
+        ctr.count = 2048.0
+
+        formatted = ctr.format(elapsed=2, width=80)
+        self.assertEqual(formatted, '2.00 KiB / 3.20 MiB | 1.00 KiB/s | 0.00 s/B')
+
+        # Counter_format
+        counter_format = u'{count:!.2j}B | {rate:!.2j}B/s | {interval:!.2j} s/B'
+
+        ctr = Counter(stream=self.tty.stdout, counter_format=counter_format)
+        ctr.count = 2048.0
+
+        formatted = ctr.format(elapsed=2, width=80)
+        self.assertEqual(formatted, '2.00 KiB | 1.00 KiB/s | 0.00 s/B')
+
     def test_color(self):
         """
         Only bar characters should be colorized
@@ -554,7 +594,6 @@ class TestCounter(TestCase):
         """
         When subcounter is present, bar will be drawn in multiple colors
         """
-
         ctr = Counter(stream=self.tty.stdout, total=100, bar_format=u'{bar}')
         terminal = ctr.manager.term
         ctr.count = 50
@@ -572,6 +611,39 @@ class TestCounter(TestCase):
         formatted = ctr.format(elapsed=5, width=80)
         self.assertEqual(formatted, u'35 35.0 | 5 5.0 1.0 01:35 | 10 10.0 | 15 15.0')
 
+    def test_subcounter_field_not_populated(self):
+        """
+        Exception raised when reserved field is invalid
+        """
+
+        ctr = Counter(stream=self.tty.stdout, total=100, count=50)
+        ctr.add_subcounter('yellow')
+        ctr.add_subcounter('blue', count=10)
+
+        ctr.bar_format = u'{count_3}'
+        with self.assertRaisesRegex(ValueError, 'subcounter 3 is not defined'):
+            ctr.format(elapsed=5, width=80)
+
+        for fmt in (u'{rate_1}', u'{eta_1}', u'{interval_1}'):
+            ctr.bar_format = fmt
+            with self.assertRaisesRegex(ValueError, "'all_fields' not specified for subcounter"):
+                ctr.format(elapsed=5, width=80)
+
+        ctr = Counter(stream=self.tty.stdout, total=100, count=50,)
+        for fmt in (u'{count_0}', u'{rate_0}', u'{percentage_0}',
+                    u'{count_00}', u'{rate_00}', u'{percentage_00}'):
+            ctr.bar_format = fmt
+            with self.assertRaisesRegex(ValueError, 'no subcounters are configured'):
+                ctr.format(elapsed=5, width=80)
+
+        # Counter fields
+        ctr = Counter(stream=self.tty.stdout, count=50)
+        ctr.add_subcounter('yellow')
+        for field in ('percentage_1', 'eta_1'):
+            ctr.counter_format = u'|{%s}|' % field
+            with self.assertRaisesRegex(ValueError, 'unavailable for counter_format'):
+                ctr.format(elapsed=5, width=80)
+
     def test_subcounter_count_gt_total(self):
         """
         When total is exceeded, subcounter fields are still populated
@@ -602,6 +674,40 @@ class TestCounter(TestCase):
         bartext = term.red(BLOCK*2) + term.blue(BLOCK*3) + term.yellow(BLOCK*35) + ' ' * 40
         self.assertEqual(formatted, bartext)
 
+    def test_subcounter_prefixed(self):
+        """
+        Subcounter float fields should support prefixed formatting
+        """
+
+        bar_format = (u'{count:!.2j}B / {total:!.2j}B | {rate:!.2j}B/s | {interval:!.2j} s/B'
+                      u' | {count_0:!.2j}B | {count_00:!.2j}B'
+                      u' | {count_1:!.2j}B | {rate_1:!.2j}B/s | {interval_1:!.2j} s/B'
+                      u' | {count_2:!.2j}B | {rate_2:!.2j}B/s | {interval_2:!.2j} s/B'
+                      u' | {count_3:!.2j}B | {rate_3:!.2j}B/s | {interval_3:!.2j} s/B'
+                      )
+        ctr = Counter(stream=self.tty.stdout, total=3.2 * 2 ** 20,
+                      bar_format=bar_format, all_fields=True)
+
+        ctr.count = 2.0 ** 20
+        subcounter1 = ctr.add_subcounter('yellow')
+        subcounter2 = ctr.add_subcounter('blue')
+        subcounter3 = ctr.add_subcounter('red')
+
+        subcounter1.count = 512.0 * 2 ** 10
+        subcounter2.count = 256.0 * 2 ** 10
+        subcounter3.count = 128.0 * 2 ** 10
+
+        formatted = ctr.format(elapsed=1, width=80)
+        self.assertEqual(
+            formatted, (
+                '1.00 MiB / 3.20 MiB | 1.00 MiB/s | 0.00 s/B'
+                ' | 128.00 KiB | 896.00 KiB'
+                ' | 512.00 KiB | 512.00 KiB/s | 0.00 s/B'
+                ' | 256.00 KiB | 256.00 KiB/s | 0.00 s/B'
+                ' | 128.00 KiB | 128.00 KiB/s | 0.00 s/B'
+            )
+        )
+
     def test_close(self):
         manager = MockManager()
 
@@ -652,6 +758,26 @@ class TestCounter(TestCase):
         self.assertEqual(subcounter2.count, 5)
         self.assertTrue(subcounter2.all_fields)
 
+    def test_reserve_field_unavailable(self):
+        """
+        Exception raised when reserved field is invalid
+        """
+
+        msg1 = "Reserve field '%s' specified in format, but unavailable for bar_format"
+        msg2 = "Reserve field '%s' specified in format, but unavailable for counter_format"
+
+        ctr = Counter(stream=self.tty.stdout, total=100, count=50)
+        ctr.bar_format = u'{fill}'
+        with self.assertRaisesRegex(ValueError, msg1 % 'fill'):
+            ctr.format(elapsed=5, width=80)
+
+        ctr = Counter(stream=self.tty.stdout, count=50)
+
+        for field in ('bar', 'eta', 'percentage'):
+            ctr.counter_format = u'{%s}' % field
+            with self.assertRaisesRegex(ValueError, msg2 % field):
+                ctr.format(elapsed=5, width=80)
+
     def test_additional_fields(self):
         """
         Add additional fields to format
@@ -810,7 +936,7 @@ class TestCounter(TestCase):
         Ensure all built-in fields are populated as expected
         """
 
-        bar_fields = tuple(field for field in enlighten._counter.COUNTER_FIELDS if field != 'fill')
+        bar_fields = tuple(field for field in enlighten._counter.RESERVED_FIELDS if field != 'fill')
         bar_format = u', '.join(u'%s: {%s}' % (field, field) for field in sorted(bar_fields))
 
         ctr = Counter(stream=self.tty.stdout, total=100, bar_format=bar_format,
